@@ -9,7 +9,7 @@ use bee_message::output::OutputId;
 use std::ops::Deref;
 
 /// Represents an output id as unspent.
-#[derive(Debug, Eq, PartialEq)]
+#[derive(Eq, PartialEq)]
 pub struct Unspent(OutputId);
 
 impl From<OutputId> for Unspent {
@@ -38,6 +38,18 @@ impl Unspent {
     }
 }
 
+impl core::fmt::Display for Unspent {
+    fn fmt(&self, f: &mut core::fmt::Formatter) -> core::fmt::Result {
+        write!(f, "{}", *self)
+    }
+}
+
+impl core::fmt::Debug for Unspent {
+    fn fmt(&self, f: &mut core::fmt::Formatter) -> core::fmt::Result {
+        write!(f, "Unspent({})", self)
+    }
+}
+
 impl Packable for Unspent {
     type Error = Error;
 
