@@ -11,4 +11,4 @@ pub(crate) mod worker;
 
 pub use metadata::WhiteFlagMetadata;
 pub use white_flag::white_flag;
-pub use worker::{ConsensusWorker, ConsensusWorkerEvent};
+pub use worker::{ConsensusWorker, ConsensusWorkerCommand};
