@@ -6,13 +6,9 @@ use crate::{
     workers::{
         consensus::{metadata::WhiteFlagMetadata, state::validate_ledger_state, white_flag},
         error::Error,
-        event::{MilestoneConfirmed, OutputConsumed, OutputCreated},
-        pruning::{
-            condition::{should_prune, should_snapshot},
-            config::PruningConfig,
-            constants::{PRUNING_THRESHOLD, SOLID_ENTRY_POINT_THRESHOLD_FUTURE, SOLID_ENTRY_POINT_THRESHOLD_PAST},
-        },
-        snapshot::{config::SnapshotConfig, worker::SnapshotWorker},
+        event::{MessageReferenced, MilestoneConfirmed, OutputConsumed, OutputCreated},
+        pruning::{condition::should_prune, config::PruningConfig, prune},
+        snapshot::{condition::should_snapshot, config::SnapshotConfig, worker::SnapshotWorker},
         storage::{self, StorageBackend},
     },
 };
@@ -28,14 +24,16 @@ use bee_runtime::{event::Bus, node::Node, shutdown_stream::ShutdownStream, worke
 use bee_tangle::{ConflictReason, MsTangle, TangleWorker};
 
 use async_trait::async_trait;
-
 use futures::{channel::oneshot, stream::StreamExt};
-use log::{error, info, warn};
+use log::{debug, error, info, warn};
 use tokio::sync::mpsc;
 use tokio_stream::wrappers::UnboundedReceiverStream;
 
 use std::{any::TypeId, convert::TryInto};
 
+pub(crate) const EXTRA_SNAPSHOT_DEPTH: u32 = 5;
+pub(crate) const EXTRA_PRUNING_DEPTH: u32 = 5;
+
 /// Commands of the consensus worker.
 #[allow(clippy::type_complexity)]
 pub enum ConsensusWorkerCommand {
@@ -167,7 +165,7 @@ where
                 milestone.essence().index(),
                 milestone_id,
                 receipt,
-                storage::fetch_unspent_treasury_output(storage).await?,
+                storage::fetch_unspent_treasury_output(storage)?,
             )
             .await?,
         )
@@ -182,8 +180,7 @@ where
         &metadata.consumed_outputs,
         &metadata.balance_diffs,
         &migration,
-    )
-    .await?;
+    )?;
 
     *ledger_index = LedgerIndex(milestone.essence().index());
     tangle.update_confirmed_milestone_index(milestone.essence().index());
@@ -195,6 +192,9 @@ where
                 message_metadata.reference(milestone.essence().timestamp());
             })
             .await;
+        bus.dispatch(MessageReferenced {
+            message_id: *message_id,
+        });
     }
 
     for (message_id, conflict) in metadata.excluded_conflicting_messages.iter() {
@@ -204,6 +204,9 @@ where
                 message_metadata.reference(milestone.essence().timestamp());
             })
             .await;
+        bus.dispatch(MessageReferenced {
+            message_id: *message_id,
+        });
     }
 
     for message_id in metadata.included_messages.iter() {
@@ -213,6 +216,9 @@ where
                 message_metadata.reference(milestone.essence().timestamp());
             })
             .await;
+        bus.dispatch(MessageReferenced {
+            message_id: *message_id,
+        });
     }
 
     info!(
@@ -240,13 +246,19 @@ where
         receipt: migration.is_some(),
     });
 
-    for (_, created_output) in metadata.created_outputs {
-        bus.dispatch(OutputCreated { output: created_output });
+    for (output_id, created_output) in metadata.created_outputs {
+        bus.dispatch(OutputCreated {
+            message_id: *created_output.message_id(),
+            output_id,
+            output: created_output.inner().clone(),
+        });
     }
 
-    for (_, (_, consumed_output)) in metadata.consumed_outputs {
+    for (output_id, (created_output, _consumed_output)) in metadata.consumed_outputs {
         bus.dispatch(OutputConsumed {
-            output: consumed_output,
+            message_id: *created_output.message_id(),
+            output_id,
+            output: created_output.inner().clone(),
         });
     }
 
@@ -272,32 +284,37 @@ where
         let storage = node.storage();
         let bus = node.bus();
 
-        validate_ledger_state(&*storage).await?;
+        validate_ledger_state(&*storage)?;
 
-        let depth = if snapshot_config.depth() < SOLID_ENTRY_POINT_THRESHOLD_FUTURE {
+        let bmd = tangle.config().below_max_depth();
+
+        let snapshot_depth_min = bmd + EXTRA_SNAPSHOT_DEPTH;
+        let snapshot_depth = if snapshot_config.depth() < snapshot_depth_min {
             warn!(
-                "Configuration value for \"depth\" is too low ({}), value changed to {}.",
+                "Configuration value for \"snapshot.depth\" is too low ({}), value changed to {}.",
                 snapshot_config.depth(),
-                SOLID_ENTRY_POINT_THRESHOLD_FUTURE
+                snapshot_depth_min
             );
-            SOLID_ENTRY_POINT_THRESHOLD_FUTURE
+            snapshot_depth_min
         } else {
             snapshot_config.depth()
         };
-        let delay_min = snapshot_config.depth() + SOLID_ENTRY_POINT_THRESHOLD_PAST + PRUNING_THRESHOLD + 1;
-        let delay = if pruning_config.delay() < delay_min {
+
+        let snapshot_pruning_delta = bmd + EXTRA_PRUNING_DEPTH;
+        let pruning_delay_min = snapshot_depth + snapshot_pruning_delta;
+        let pruning_delay = if pruning_config.delay() < pruning_delay_min {
             warn!(
-                "Configuration value for \"delay\" is too low ({}), value changed to {}.",
+                "Configuration value for \"pruning.delay\" is too low ({}), value changed to {}.",
                 pruning_config.delay(),
-                delay_min
+                pruning_delay_min
             );
-            delay_min
+            pruning_delay_min
         } else {
             pruning_config.delay()
         };
 
         // Unwrap is fine because ledger index was already in storage or just added by the snapshot worker.
-        let mut ledger_index = storage::fetch_ledger_index(&*storage).await?.unwrap();
+        let mut ledger_index = storage::fetch_ledger_index(&*storage)?.unwrap();
         let mut receipt_migrated_at = MilestoneIndex(0);
 
         node.spawn::<Self, _, _>(|shutdown| async move {
@@ -326,35 +343,46 @@ where
                             continue;
                         }
 
-                        if should_snapshot(&tangle, MilestoneIndex(*ledger_index), depth, &snapshot_config) {
-                            // TODO
-                            // if let Err(e) = snapshot(snapshot_config.path(), event.index - depth) {
-                            //     error!("Failed to create snapshot: {:?}.", e);
-                            // }
+                        match should_snapshot(&tangle, ledger_index, snapshot_depth, &snapshot_config) {
+                            Ok(()) => {
+                                // TODO
+                                // if let Err(e) = snapshot(snapshot_config.path(), event.index - snapshot_depth) {
+                                //     error!("Failed to create snapshot: {:?}.", e);
+                                // }
+                            }
+                            Err(reason) => {
+                                debug!("Snapshotting skipped: {:?}", reason);
+                            }
                         }
 
-                        if should_prune(&tangle, MilestoneIndex(*ledger_index), delay, &pruning_config) {
-                            // TODO
-                            // if let Err(e) = prune_database(&tangle, MilestoneIndex(*event.index - delay)) {
-                            //     error!("Failed to prune database: {:?}.", e);
-                            // }
+                        match should_prune(&tangle, ledger_index, pruning_delay, &pruning_config) {
+                            Ok((start_index, target_index)) => {
+                                if let Err(e) =
+                                    prune::prune(&tangle, &storage, &bus, start_index, target_index, &pruning_config)
+                                        .await
+                                {
+                                    error!("Pruning failed: {:?}.", e);
+                                }
+                            }
+                            Err(reason) => {
+                                debug!("Pruning skipped: {:?}", reason);
+                            }
                         }
                     }
                     ConsensusWorkerCommand::FetchBalance(address, sender) => {
-                        if let Err(e) = sender.send((storage::fetch_balance(&*storage, &address).await, ledger_index)) {
+                        if let Err(e) = sender.send((storage::fetch_balance(&*storage, &address), ledger_index)) {
                             error!("Error while sending balance: {:?}", e);
                         }
                     }
                     ConsensusWorkerCommand::FetchOutput(output_id, sender) => {
-                        if let Err(e) = sender.send((storage::fetch_output(&*storage, &output_id).await, ledger_index))
-                        {
+                        if let Err(e) = sender.send((storage::fetch_output(&*storage, &output_id), ledger_index)) {
                             error!("Error while sending output: {:?}", e);
                         }
                     }
                     ConsensusWorkerCommand::FetchOutputs(address, sender) => match address {
                         Address::Ed25519(address) => {
                             if let Err(e) = sender.send((
-                                storage::fetch_outputs_for_ed25519_address(&*storage, &address).await,
+                                storage::fetch_outputs_for_ed25519_address(&*storage, &address),
                                 ledger_index,
                             )) {
                                 error!("Error while sending output: {:?}", e);
